@@ -9,6 +9,17 @@ class RunningStats:
     def __init__(self) -> None:
         self.clear()
 
+    def __add__(self: "RunningStats", other: "RunningStats") -> "RunningStats":
+        s = RunningStats()
+        if other.n > 0:
+            s.m1 = (self.m1 * self.n + other.m1 * other.n) / (self.n + other.n)
+            # TBD: Fix s.m2 and friends
+            # For now, leave at zero.
+            s.n = self.n + other.n
+        else:
+            s = self
+        return s
+
     def clear(self) -> None:
         """Reset for new samples"""
         self.n = 0
