@@ -1,10 +1,9 @@
 import dis
-
 from functools import lru_cache
 from types import CodeType
 from typing import FrozenSet
 
-from scalene.scalene_statistics import *
+from scalene.scalene_statistics import ByteCodeIndex
 
 
 class ScaleneFuncUtils:
@@ -25,6 +24,10 @@ class ScaleneFuncUtils:
     @lru_cache(maxsize=None)
     def is_call_function(code: CodeType, bytei: ByteCodeIndex) -> bool:
         """Returns true iff the bytecode at the given index is a function call."""
-        return any((ins.offset == bytei
-                    and ins.opcode in ScaleneFuncUtils.__call_opcodes)
-                   for ins in dis.get_instructions(code))
+        return any(
+            (
+                ins.offset == bytei
+                and ins.opcode in ScaleneFuncUtils.__call_opcodes
+            )
+            for ins in dis.get_instructions(code)
+        )
