@@ -1,17 +1,19 @@
 import contextlib
 
 with contextlib.suppress(Exception):
+    import sys
+    from typing import Any
+
     from IPython.core.magic import (
         Magics,
-        magics_class,
-        line_magic,
         line_cell_magic,
+        line_magic,
+        magics_class,
     )
+
     from scalene import scalene_profiler
     from scalene.scalene_arguments import ScaleneArguments
     from scalene.scalene_parseargs import ScaleneParseArgs
-    from typing import Any
-    import sys
 
     @magics_class
     class ScaleneMagics(Magics):  # type: ignore
@@ -27,7 +29,8 @@ with contextlib.suppress(Exception):
             with open(filename, "w+") as tmpfile:
                 tmpfile.write(newcode)
             args.cpu_only = True  # full Scalene is not yet working, force to use CPU-only mode
-            scalene_profiler.Scalene.run_profiler(args, [filename])
+            scalene_profiler.Scalene.set_initialized()
+            scalene_profiler.Scalene.run_profiler(args, [filename], is_jupyter=True)
 
         @line_cell_magic
         def scalene(self, line: str, cell: str = "") -> None:
@@ -69,3 +72,12 @@ with contextlib.suppress(Exception):
                 )
             )
         )
+        if sys.platform == "darwin":
+            print()
+            print(
+                "\n".join(
+                    textwrap.wrap(
+                        "NOTE: in Jupyter notebook on MacOS, Scalene cannot profile child processes. Do not run to try Scalene with multiprocessing in Jupyter Notebook"
+                    )
+                )
+            )
