@@ -1,23 +1,21 @@
-import shutil
 import sys
 import tempfile
-
-from collections import defaultdict, OrderedDict
+from collections import OrderedDict, defaultdict
 from operator import itemgetter
 from pathlib import Path
+from typing import Any, Callable, Dict, List, Union
+
+from rich import box
 from rich.console import Console
 from rich.markdown import Markdown
 from rich.syntax import Syntax
 from rich.table import Table
 from rich.text import Text
-from rich import box
 
 from scalene import sparkline
 from scalene.scalene_json import ScaleneJSON
-from scalene.syntaxline import SyntaxLine
 from scalene.scalene_statistics import Filename, LineNumber, ScaleneStatistics
-
-from typing import Any, Callable, Dict, List, Union
+from scalene.syntaxline import SyntaxLine
 
 
 class ScaleneOutput:
@@ -25,6 +23,9 @@ class ScaleneOutput:
     # Threshold for highlighting lines of code in red.
     highlight_percentage = 33
 
+    # Color for highlighted text (over the threshold of CPU time)
+    highlight_color = "bold red"
+
     # Default threshold for percent of CPU time to report a file.
     cpu_percent_threshold = 1
 
@@ -42,6 +43,40 @@ class ScaleneOutput:
         self.gpu = False
 
     # Profile output methods
+
+    def output_top_memory(
+        self, title: str, console: Console, mallocs: Dict[LineNumber, float]
+    ) -> None:
+        # Print the top N lines by memory consumption, as long
+        # as they are above some threshold MB in size.
+        print_top_mallocs_count = 5
+        print_top_mallocs_threshold_mb = 1
+        if len(mallocs) > 0:
+            printed_header = False
+            number = 1
+            for malloc_lineno in mallocs:
+                # Don't print lines with less than the threshold MB allocated.
+                if mallocs[malloc_lineno] <= print_top_mallocs_threshold_mb:
+                    break
+                # Only print the top N.
+                if number > print_top_mallocs_count:
+                    break
+                # Print the header only if we are printing something (and only once).
+                if not printed_header:
+                    console.print(title)
+                    printed_header = True
+                output_str = (
+                    "("
+                    + str(number)
+                    + ") "
+                    + ("%5.0f" % (malloc_lineno))
+                    + ": "
+                    + ("%5.0f" % (mallocs[malloc_lineno]))
+                    + " MB"
+                )
+                console.print(Markdown(output_str, style="dark_green"))
+                number += 1
+
     def output_profile_line(
         self,
         json: ScaleneJSON,
@@ -68,9 +103,9 @@ class ScaleneOutput:
         )
         if not obj:
             return False
-        if -1 < obj["n_growth_mb"] < 1:
+        if -1 < obj["n_peak_mb"] < 1:
             # Don't print out "-0" or anything below 1.
-            obj["n_growth_mb"] = 0
+            obj["n_peak_mb"] = 0
 
         # Finally, print results.
         n_cpu_percent_c_str: str = (
@@ -89,17 +124,17 @@ class ScaleneOutput:
             else f"{obj['n_cpu_percent_python']:5.0f}%"
         )
         n_growth_mem_str = ""
-        if obj["n_growth_mb"] < 1024:
+        if obj["n_peak_mb"] < 1024:
             n_growth_mem_str = (
                 ""
-                if (not obj["n_growth_mb"] and not obj["n_usage_fraction"])
-                else f"{obj['n_growth_mb']:5.0f}M"
+                if (not obj["n_peak_mb"] and not obj["n_usage_fraction"])
+                else f"{obj['n_peak_mb']:5.0f}M"
             )
         else:
             n_growth_mem_str = (
                 ""
-                if (not obj["n_growth_mb"] and not obj["n_usage_fraction"])
-                else f"{(obj['n_growth_mb'] / 1024):5.2f}G"
+                if (not obj["n_peak_mb"] and not obj["n_usage_fraction"])
+                else f"{(obj['n_peak_mb'] / 1024):5.2f}G"
             )
 
         n_usage_fraction_str: str = (
@@ -154,12 +189,18 @@ class ScaleneOutput:
                 )
                 >= self.highlight_percentage
             ):
-                ncpps = Text.assemble((n_cpu_percent_python_str, "bold red"))
-                ncpcs = Text.assemble((n_cpu_percent_c_str, "bold red"))
+                ncpps = Text.assemble(
+                    (n_cpu_percent_python_str, self.highlight_color)
+                )
+                ncpcs = Text.assemble(
+                    (n_cpu_percent_c_str, self.highlight_color)
+                )
                 nufs = Text.assemble(
-                    (spark_str + n_usage_fraction_str, "bold red")
+                    (spark_str + n_usage_fraction_str, self.highlight_color)
+                )
+                ngpus = Text.assemble(
+                    (n_gpu_percent_str, self.highlight_color)
                 )
-                ngpus = Text.assemble((n_gpu_percent_str, "bold red"))
             else:
                 ncpps = n_cpu_percent_python_str
                 ncpcs = n_cpu_percent_c_str
@@ -204,9 +245,15 @@ class ScaleneOutput:
                 + obj["n_cpu_percent_python"]
                 + obj["n_gpu_percent"]
             ) >= self.highlight_percentage:
-                ncpps = Text.assemble((n_cpu_percent_python_str, "bold red"))
-                ncpcs = Text.assemble((n_cpu_percent_c_str, "bold red"))
-                ngpus = Text.assemble((n_gpu_percent_str, "bold red"))
+                ncpps = Text.assemble(
+                    (n_cpu_percent_python_str, self.highlight_color)
+                )
+                ncpcs = Text.assemble(
+                    (n_cpu_percent_c_str, self.highlight_color)
+                )
+                ngpus = Text.assemble(
+                    (n_gpu_percent_str, self.highlight_color)
+                )
             else:
                 ncpps = n_cpu_percent_python_str
                 ncpcs = n_cpu_percent_c_str
@@ -237,6 +284,7 @@ class ScaleneOutput:
 
     def output_profiles(
         self,
+        column_width: int,
         stats: ScaleneStatistics,
         pid: int,
         profile_this_code: Callable[[Filename, LineNumber], bool],
@@ -307,19 +355,6 @@ class ScaleneOutput:
 
         null = tempfile.TemporaryFile(mode="w+")
 
-        # Get column width of the terminal and adjust to fit.
-        # Note that Scalene works best with at least 132 columns.
-        column_width = 132
-        if not self.html:
-            try:
-                # If we are in a Jupyter notebook, stick with 132
-                if "ipykernel" in sys.modules:
-                    column_width = 132
-                else:
-                    column_width = shutil.get_terminal_size().columns
-            except:
-                pass
-
         console = Console(
             width=column_width,
             record=True,
@@ -434,7 +469,7 @@ class ScaleneOutput:
                     width=7,
                 )
                 tbl.add_column(
-                    Markdown("––––––  \n_avg_", style="dark_green"),
+                    Markdown("––––––  \n_peak_", style="dark_green"),
                     style="dark_green",
                     no_wrap=True,
                     width=6,
@@ -445,7 +480,7 @@ class ScaleneOutput:
                     ),
                     style="dark_green",
                     no_wrap=True,
-                    width=14,
+                    width=15,
                 )
                 tbl.add_column(
                     Markdown("Copy  \n_(MB/s)_", style="yellow4"),
@@ -523,14 +558,23 @@ class ScaleneOutput:
             # Potentially print a function summary.
             fn_stats = stats.build_function_stats(fname)
             print_fn_summary = False
-            for fn_name in fn_stats.cpu_samples_python:
+            # Check CPU samples and memory samples.
+            all_samples = set()
+            all_samples |= set(fn_stats.cpu_samples_python.keys())
+            all_samples |= set(fn_stats.cpu_samples_c.keys())
+            all_samples |= set(fn_stats.memory_malloc_samples.keys())
+            all_samples |= set(fn_stats.memory_free_samples.keys())
+            for fn_name in all_samples:
                 if fn_name == fname:
                     continue
                 print_fn_summary = True
                 break
 
             if print_fn_summary:
-                tbl.add_row(None, end_section=True)
+                try:
+                    tbl.add_row(None, end_section=True)
+                except TypeError:  # rich < 9.4.0 compatibility
+                    tbl.add_row(None)
                 txt = Text.assemble(
                     f"function summary for {fname}", style="bold italic"
                 )
@@ -577,57 +621,51 @@ class ScaleneOutput:
 
             console.print(tbl)
 
-            # Report top K lines (currently 5) in terms of average memory consumption.
+            # Compute AVERAGE memory consumption.
             avg_mallocs: Dict[LineNumber, float] = defaultdict(float)
             for line_no in stats.bytei_map[fname]:
-                for bytecode_index in stats.bytei_map[fname][line_no]:
-                    avg_mallocs[line_no] += (
-                        stats.memory_malloc_samples[fname][line_no][
-                            bytecode_index
-                        ]
-                        #- stats.memory_free_samples[fname][line_no][
-                        #    bytecode_index
-                        #]
-                    )
-                    count = stats.memory_malloc_count[fname][line_no][bytecode_index]
-                    if count:
-                        avg_mallocs[line_no] /= count
+                n_malloc_mb = stats.memory_aggregate_footprint[fname][
+                    line_no
+                ]  # 0.0
+                # for bytei in stats.memory_malloc_samples[fname][line_no]:
+                #    n_malloc_mb += stats.memory_malloc_samples[fname][line_no][
+                #        bytei
+                #    ]
+                count = stats.memory_malloc_count[fname][line_no]
+                if count:
+                    avg_mallocs[line_no] = n_malloc_mb / count
+                else:
+                    # Setting to n_malloc_mb addresses the edge case where this allocation is the last line executed.
+                    avg_mallocs[line_no] = n_malloc_mb
 
             avg_mallocs = OrderedDict(
                 sorted(avg_mallocs.items(), key=itemgetter(1), reverse=True)
             )
-            # Print the top N lines by net memory consumption, as long
+
+            # Compute (really, aggregate) PEAK memory consumption.
+            peak_mallocs: Dict[LineNumber, float] = defaultdict(float)
+            for line_no in stats.bytei_map[fname]:
+                peak_mallocs[line_no] = stats.memory_max_footprint[fname][
+                    line_no
+                ]
+
+            peak_mallocs = OrderedDict(
+                sorted(peak_mallocs.items(), key=itemgetter(1), reverse=True)
+            )
+
+            # Print the top N lines by AVERAGE memory consumption, as long
             # as they are above some threshold MB in size.
-            print_top_mallocs_count = 5
-            print_top_mallocs_threshold_mb = 1
-            if len(avg_mallocs) > 0:
-                printed_header = False
-                number = 1
-                for avg_malloc_lineno in avg_mallocs:
-                    # Don't print lines with less than the threshold MB allocated.
-                    if (
-                        avg_mallocs[avg_malloc_lineno]
-                        <= print_top_mallocs_threshold_mb
-                    ):
-                        break
-                    # Only print the top N.
-                    if number > print_top_mallocs_count:
-                        break
-                    # Print the header only if we are printing something (and only once).
-                    if not printed_header:
-                        console.print("Top average memory consumption, by line:")
-                        printed_header = True
-                    output_str = (
-                        "("
-                        + str(number)
-                        + ") "
-                        + ("%5.0f" % (avg_malloc_lineno))
-                        + ": "
-                        + ("%5.0f" % (avg_mallocs[avg_malloc_lineno]))
-                        + " MB"
-                    )
-                    console.print(Markdown(output_str, style="dark_green"))
-                    number += 1
+            self.output_top_memory(
+                "Top AVERAGE memory consumption, by line:",
+                console,
+                avg_mallocs,
+            )
+
+            # Print the top N lines by PEAK memory consumption, as long
+            # as they are above some threshold MB in size.
+            self.output_top_memory(
+                "Top PEAK memory consumption, by line:", console, peak_mallocs
+            )
 
             # Only report potential leaks if the allocation velocity (growth rate) is above some threshold
             # FIXME: fixed at 1% for now.
@@ -643,13 +681,14 @@ class ScaleneOutput:
                     allocs = item[0]
                     expected_leak = (frees + 1) / (frees + allocs + 2)
                     if expected_leak <= leak_reporting_threshold:
-                        leaks.append(
-                            (
-                                keys[index],
-                                1 - expected_leak,
-                                avg_mallocs[keys[index]],
+                        if keys[index] in avg_mallocs:
+                            leaks.append(
+                                (
+                                    keys[index],
+                                    1 - expected_leak,
+                                    avg_mallocs[keys[index]],
+                                )
                             )
-                        )
                 if len(leaks) > 0:
                     # Report in descending order by least likelihood
                     for leak in sorted(leaks, key=itemgetter(1), reverse=True):
