@@ -1,17 +1,11 @@
-from scalene.scalene_arguments import ScaleneArguments
-from scalene.scalene_version import scalene_version
-
-from typing import (
-    Any,
-    List,
-    NoReturn,
-    Optional,
-    Tuple,
-)
-from textwrap import dedent
 import argparse
 import contextlib
 import sys
+from textwrap import dedent
+from typing import Any, List, NoReturn, Optional, Tuple
+
+from scalene.scalene_arguments import ScaleneArguments
+from scalene.scalene_version import scalene_version
 
 
 class RichArgParser(argparse.ArgumentParser):
@@ -93,6 +87,13 @@ for the process ID that Scalene reports. For example:
             const=True,
             help="prints the version number for this release of Scalene and exits",
         )
+        parser.add_argument(
+            "--column-width",
+            dest="column_width",
+            type=int,
+            default=defaults.column_width,
+            help=f"Column width for profile output (default: [blue]{defaults.column_width}[/blue])",
+        )
         parser.add_argument(
             "--outfile",
             type=str,
@@ -176,6 +177,19 @@ for the process ID that Scalene reports. For example:
             )
             + "[/blue])",
         )
+        parser.add_argument(
+            "--profile-exclude",
+            dest="profile_exclude",
+            type=str,
+            default=defaults.profile_exclude,
+            help="do not profile code in filenames that contain the given strings, separated by commas (default: [blue]"
+            + (
+                "no restrictions"
+                if not defaults.profile_exclude
+                else defaults.profile_exclude
+            )
+            + "[/blue])",
+        )
         parser.add_argument(
             "--use-virtual-time",
             dest="use_virtual_time",
@@ -198,6 +212,13 @@ for the process ID that Scalene reports. For example:
             default=defaults.cpu_sampling_rate,
             help=f"CPU sampling rate (default: every [blue]{defaults.cpu_sampling_rate}s[/blue])",
         )
+        parser.add_argument(
+            "--allocation-sampling-window",
+            dest="allocation_sampling_window",
+            type=int,
+            default=defaults.allocation_sampling_window,
+            help=f"Allocation sampling window size, in bytes (default: [blue]{defaults.allocation_sampling_window} bytes[/blue])",
+        )
         parser.add_argument(
             "--malloc-threshold",
             dest="malloc_threshold",
@@ -213,6 +234,16 @@ for the process ID that Scalene reports. For example:
             default="",
             help="The directory containing the code to profile (default: [blue]the path to the profiled program[/blue])",
         )
+        parser.add_argument(
+            "--memory-leak-detector",
+            dest="memory_leak_detector",
+            action="store_true",
+            default=defaults.memory_leak_detector,
+            help="EXPERIMENTAL: report likely memory leaks (default: [blue]"
+            + (str(defaults.memory_leak_detector))
+            + "[/blue])",
+        )
+
         group = parser.add_mutually_exclusive_group(required=False)
         group.add_argument(
             "--on",
