@@ -2,20 +2,28 @@ import argparse
 import contextlib
 import os
 import platform
-import scalene
 import signal
 import struct
 import subprocess
 import sys
-
 from typing import Dict
 
+import scalene
+
 
 class ScalenePreload:
     @staticmethod
     def get_preload_environ(args: argparse.Namespace) -> Dict[str, str]:
         env = dict()
 
+        # Set allocation sampling window (sync environment variable
+        # name with src/include/sampleheap.hpp).
+        env["SCALENE_ALLOCATION_SAMPLING_WINDOW"] = str(
+            args.allocation_sampling_window
+        )
+
+        # Set environment variables for loading the Scalene dynamic library,
+        # which interposes on allocation and copying functions.
         if sys.platform == "linux":
             if not args.cpu_only:
                 env["LD_PRELOAD"] = os.path.join(
@@ -100,7 +108,6 @@ class ScalenePreload:
                 result.wait()
             except KeyboardInterrupt:
                 result.returncode = 0
-                pass
             if result.returncode < 0:
                 print(
                     "Scalene error: received signal",
