@@ -23,11 +23,10 @@ import inspect
 import json
 import math
 import multiprocessing
-import pathlib
 import os
-import random
+import pathlib
+import platform
 import re
-import resource
 import signal
 import stat
 import sys
@@ -35,27 +34,19 @@ import tempfile
 import threading
 import time
 import traceback
-
 from collections import defaultdict
 from functools import lru_cache
 from signal import Handlers, Signals
 from types import FrameType
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-    cast,
-)
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union, cast
 
 from scalene.scalene_arguments import ScaleneArguments
 from scalene.scalene_funcutils import ScaleneFuncUtils
 from scalene.scalene_json import ScaleneJSON
 from scalene.scalene_mapfile import ScaleneMapFile
+from scalene.scalene_output import ScaleneOutput
+from scalene.scalene_preload import ScalenePreload
+from scalene.scalene_signals import ScaleneSignals
 from scalene.scalene_statistics import (
     Address,
     ByteCodeIndex,
@@ -63,10 +54,16 @@ from scalene.scalene_statistics import (
     LineNumber,
     ScaleneStatistics,
 )
-from scalene.scalene_output import ScaleneOutput
-from scalene.scalene_preload import ScalenePreload
-from scalene.scalene_signals import ScaleneSignals
-from scalene.scalene_gpu import ScaleneGPU
+
+if sys.platform != "win32":
+    import resource
+
+# For now, disable experimental GPU support for Apple
+if False:  # platform.system() == "Darwin":
+    from scalene.scalene_apple_gpu import ScaleneAppleGPU as ScaleneGPU
+else:
+    from scalene.scalene_gpu import ScaleneGPU
+
 from scalene.scalene_parseargs import ScaleneParseArgs, StopJupyterExecution
 from scalene.scalene_sigqueue import ScaleneSigQueue
 
@@ -93,6 +90,9 @@ def scalene_redirect_profile(func: Any) -> Any:
 
 builtins.profile = scalene_redirect_profile  # type: ignore
 
+# Must equal src/include/sampleheap.hpp NEWLINE *minus 1*
+NEWLINE_TRIGGER_LENGTH = 98820  # SampleHeap<...>::NEWLINE-1
+
 
 def start() -> None:
     Scalene.start()
@@ -105,18 +105,13 @@ def stop() -> None:
 class Scalene:
     """The Scalene profiler itself."""
 
-    # Debugging flag, for internal use only.
-    __debug: bool = False
     # Whether the current profiler is a child
     __is_child = -1
     # the pid of the primary profiler
     __parent_pid = -1
-
-    __last_profiled: Tuple[Filename, LineNumber] = (
-        Filename("NADA"),
-        LineNumber(0),
-    )
-    __last_profiled_invalidated: bool = False
+    __initialized: bool = False
+    __last_profiled = (Filename("NADA"), LineNumber(0), ByteCodeIndex(0))
+    __last_profiled_invalidated = False
 
     # Support for @profile
     # decorated files
@@ -156,9 +151,6 @@ class Scalene:
         + str(sys.version_info.minor),
     ]
 
-    # last num seconds between interrupts for CPU sampling.
-    __last_cpu_sampling_rate: float = 0
-
     # when did we last receive a signal?
     __last_signal_time_virtual: float = 0
     __last_signal_time_wallclock: float = 0
@@ -202,50 +194,70 @@ class Scalene:
         signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        this_frame: FrameType,
+        this_frame: Optional[FrameType],
     ) -> None:
         raise KeyboardInterrupt
 
     @staticmethod
-    def invalidate_lines(frame: FrameType, _event: str, _arg: str) -> Any:
-        # Mark the last_profiled information as invalid as soon as we execute a different line of code.
-        # FIXME this only correctly supports single-threaded programs at the moment.
-        if (
-            False
-        ):  # for testing purposes only - if true, measure overhead when on all the time.
-            try:
-                return Scalene.invalidate_lines
-            except:
-                return None
-        try:
-            # Stop tracing when we've invalidated or when we're done profiling.
-            # This needs to be inside the try-except because during shutdown,
-            # the members of Scalene disappear.
-            if Scalene.__last_profiled_invalidated or Scalene.__done:
-                sys.settrace(None)
-                return None
-        except:
-            sys.settrace(None)
-            return None
+    def on_stack(
+        frame: FrameType, fname: Filename, lineno: LineNumber
+    ) -> Optional[FrameType]:
+        """Returns true iff the given filename and line number are anywhere on the stack starting at frame."""
+        found_frame = None
         f = frame
-        if f.f_code.co_filename and (f.f_code.co_filename[0] == "<" or "scalene" in f.f_code.co_filename):
-            # Don't trace this scope, since it is executing inside of Scalene or Python internals.
-            f.f_trace_lines = False
-            return None
-        if Scalene.__last_profiled != (f.f_code.co_filename, f.f_lineno):
-            # We've executed a different line. Mark this as
-            # invalidated and stop tracing IF we aren't still executing
-            # the last-profiled line at a higher scope.
-            while f:
-                if (f.f_code.co_filename, f.f_lineno) == Scalene.__last_profiled:
-                    # Still on the stack; keep tracing.
-                    return Scalene.invalidate_lines
-                f = cast(FrameType, f.f_back)
-            # Not on the stack anywhere - we are done tracing.
-            Scalene.__last_profiled_invalidated = True
+        while f:
+            if (f.f_code.co_filename, f.f_lineno) == (fname, lineno):
+                found_frame = f
+                break
+            f = cast(FrameType, f.f_back)
+        return found_frame
+
+    @staticmethod
+    def update_line() -> None:
+        # Mark a new line by allocating the trigger number of bytes.
+        bytearray(NEWLINE_TRIGGER_LENGTH)
+
+    @staticmethod
+    def invalidate_lines(frame: FrameType, _event: str, _arg: str) -> Any:
+        """Mark the last_profiled information as invalid as soon as we execute a different line of code."""
+        try:
+            # If we are still on the same line, return.
+            ff = frame.f_code.co_filename
+            fl = frame.f_lineno
+            (fname, lineno, lasti) = Scalene.__last_profiled
+            if (ff == fname) and (fl == lineno):
+                return None
+            # Different line: stop tracing this frame.
+            frame.f_trace = None
+            frame.f_trace_lines = False
+            # If we are not in a file we should be tracing, return.
+            if not Scalene.should_trace(ff):
+                return None
+            # Check if we are still executing the same line of code or
+            # not (whether in this frame or one above it).
+            f = Scalene.on_stack(frame, fname, lineno)
+            if f:
+                # Still the same line, but somewhere up the stack
+                # (since we returned when it was the same line in this
+                # frame). Stop tracing in this frame.
+                return None
+            # We are on a different line; stop tracing and increment the count.
             sys.settrace(None)
+            Scalene.update_line()
+            Scalene.__last_profiled_invalidated = False
+            Scalene.__last_profiled = (
+                Filename(ff),
+                LineNumber(fl),
+                ByteCodeIndex(frame.f_lasti),
+            )
+            return None
+        except AttributeError:
+            # This can happen when Scalene shuts down.
+            return None
+        except Exception as e:
+            print("Error in program being profiled:\n", e)
+            traceback.print_exc()
             return None
-        return Scalene.invalidate_lines
 
     @classmethod
     def clear_metrics(cls) -> None:
@@ -262,7 +274,11 @@ class Scalene:
 
     @classmethod
     def remove_child_pid(cls, pid: int) -> None:
-        cls.__child_pids.remove(pid)
+        try:
+            cls.__child_pids.remove(pid)
+        except KeyError:
+            # Defensive programming: this should never happen.
+            pass
 
     # Replacement @profile decorator function.
     # We track which functions - in which files - have been decorated,
@@ -295,6 +311,7 @@ class Scalene:
         # Returns the function itself to the calling file for the sake
         # of not displaying unusual errors if someone attempts to call
         # it
+
         @functools.wraps(func)
         def wrapped(*args: Any, **kwargs: Any) -> Any:
             return func(*args, **kwargs)  # type: ignore
@@ -341,21 +358,53 @@ class Scalene:
         signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        this_frame: FrameType,
+        this_frame: Optional[FrameType],
     ) -> None:
-        Scalene.__alloc_sigq.put((signum, this_frame))
+        invalidated = Scalene.__last_profiled_invalidated
+        (fname, lineno, lasti) = Scalene.__last_profiled
+        Scalene.enter_function_meta(this_frame, Scalene.__stats)
+        # Walk the stack till we find a line of code in a file we are tracing.
+        found_frame = False
+        f = this_frame
+        while f:
+            if Scalene.should_trace(f.f_code.co_filename):
+                found_frame = True
+                break
+            f = cast(FrameType, f.f_back)
+        if not found_frame:
+            return
+        assert f
+        # Start tracing until we execute a different line of
+        # code in a file we are tracking.
+        # First, see if we have now executed a different line of code.
+        # If so, increment.
+        if invalidated or not (
+            fname == Filename(f.f_code.co_filename)
+            and lineno == LineNumber(f.f_lineno)
+        ):
+            Scalene.update_line()
+        Scalene.__last_profiled_invalidated = False
+        Scalene.__last_profiled = (
+            Filename(f.f_code.co_filename),
+            LineNumber(f.f_lineno),
+            ByteCodeIndex(f.f_lasti),
+        )
+        Scalene.__alloc_sigq.put([0])
+        # Start tracing.
+        sys.settrace(Scalene.invalidate_lines)
+        f.f_trace = Scalene.invalidate_lines
+        f.f_trace_lines = True
         del this_frame
-        if not Scalene.__last_profiled_invalidated:  # and not sys.gettrace():
-            sys.settrace(Scalene.invalidate_lines)
 
     @staticmethod
     def free_signal_handler(
         signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        this_frame: FrameType,
+        this_frame: Optional[FrameType],
     ) -> None:
-        Scalene.__alloc_sigq.put((signum, this_frame))
+        Scalene.enter_function_meta(this_frame, Scalene.__stats)
+        Scalene.__alloc_sigq.put([0])
         del this_frame
 
     @staticmethod
@@ -363,7 +412,7 @@ class Scalene:
         signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        this_frame: FrameType,
+        this_frame: Optional[FrameType],
     ) -> None:
         Scalene.__memcpy_sigq.put((signum, this_frame))
         del this_frame
@@ -397,20 +446,18 @@ class Scalene:
             Scalene.memcpy_signal_handler,
         )
         # Set every signal to restart interrupted system calls.
-        signal.siginterrupt(Scalene.__signals.cpu_signal, False)
-        signal.siginterrupt(Scalene.__signals.malloc_signal, False)
-        signal.siginterrupt(Scalene.__signals.free_signal, False)
-        signal.siginterrupt(Scalene.__signals.memcpy_signal, False)
+        for s in Scalene.__signals.get_all_signals():
+            signal.siginterrupt(s, False)
         # Turn on the CPU profiling timer to run at the sampling rate (exactly once).
         signal.signal(
             Scalene.__signals.cpu_signal,
             Scalene.cpu_signal_handler,
         )
-        signal.setitimer(
-            Scalene.__signals.cpu_timer_signal,
-            Scalene.__args.cpu_sampling_rate,
-            0,
-        )
+        if sys.platform != "win32":
+            signal.setitimer(
+                Scalene.__signals.cpu_timer_signal,
+                Scalene.__args.cpu_sampling_rate,
+            )
 
     def __init__(
         self,
@@ -418,17 +465,18 @@ class Scalene:
         program_being_profiled: Optional[Filename] = None,
     ) -> None:
         # gc.set_debug(gc.DEBUG_SAVE)
-        import scalene.replacement_pjoin
+        import scalene.replacement_exit
 
         # Hijack lock, poll, thread_join, fork, and exit.
         import scalene.replacement_lock
-        import scalene.replacement_thread_join
-        import scalene.replacement_exit
         import scalene.replacement_mp_lock
+        import scalene.replacement_pjoin
+        import scalene.replacement_thread_join
+        import scalene.replacement_get_context
 
         if sys.platform != "win32":
-            import scalene.replacement_poll_selector
             import scalene.replacement_fork
+            import scalene.replacement_poll_selector
 
         Scalene.__args = cast(ScaleneArguments, arguments)
         Scalene.__cpu_sigq = ScaleneSigQueue(Scalene.cpu_sigqueue_processor)
@@ -453,7 +501,7 @@ class Scalene:
         except:
             # Ignore if we aren't profiling memory; otherwise, exit.
             if not arguments.cpu_only:
-                sys.exit(-1)
+                sys.exit(1)
 
         Scalene.__signals.set_timer_signals(arguments.use_virtual_time)
         if arguments.pid:
@@ -528,25 +576,42 @@ class Scalene:
         signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        this_frame: FrameType,
+        this_frame: Optional[FrameType],
     ) -> None:
         """Wrapper for CPU signal handlers."""
-        ru = resource.getrusage(resource.RUSAGE_SELF)
-        now_sys = ru.ru_stime
-        now_user = ru.ru_utime
+        # Get current time stats.
+        if sys.platform != "win32":
+            # On Linux/Mac, use getrusage, which provides higher
+            # resolution values than os.times() for some reason.
+            ru = resource.getrusage(resource.RUSAGE_SELF)
+            now_sys = ru.ru_stime
+            now_user = ru.ru_utime
+        else:
+            time_info = os.times()
+            now_sys = time_info.system
+            now_user = time_info.user
         now_virtual = time.process_time()
         now_wallclock = time.perf_counter()
         if (
             Scalene.__last_signal_time_virtual == 0
             or Scalene.__last_signal_time_wallclock == 0
         ):
+            # Initialization: store values and update on the next pass.
             Scalene.__last_signal_time_virtual = now_virtual
             Scalene.__last_signal_time_wallclock = now_wallclock
             Scalene.__last_signal_time_sys = now_sys
             Scalene.__last_signal_time_user = now_user
+            if sys.platform != "win32":
+                signal.setitimer(
+                    Scalene.__signals.cpu_timer_signal,
+                    Scalene.__args.cpu_sampling_rate,
+                )
+            return
+
         # Sample GPU load as well.
         gpu_load = Scalene.__gpu.load()
-        gpu_mem_used = Scalene.__gpu.memory_used()
+        gpu_mem_used = 0  # Scalene.__gpu.memory_used()
+        # Pass on to the signal queue.
         Scalene.__cpu_sigq.put(
             (
                 signum,
@@ -563,19 +628,16 @@ class Scalene:
                 Scalene.__last_signal_time_user,
             )
         )
+        # Store the latest values as the previously recorded values.
+        Scalene.__last_signal_time_virtual = now_virtual
+        Scalene.__last_signal_time_wallclock = now_wallclock
+        Scalene.__last_signal_time_sys = now_sys
+        Scalene.__last_signal_time_user = now_user
         if sys.platform != "win32":
             signal.setitimer(
                 Scalene.__signals.cpu_timer_signal,
                 Scalene.__args.cpu_sampling_rate,
-                0,
             )
-        ru = resource.getrusage(resource.RUSAGE_SELF)
-        now_sys = ru.ru_stime
-        now_user = ru.ru_utime
-        Scalene.__last_signal_time_virtual = time.process_time()
-        Scalene.__last_signal_time_wallclock = time.perf_counter()
-        Scalene.__last_signal_time_sys = now_sys
-        Scalene.__last_signal_time_user = now_user
 
     @staticmethod
     def output_profile() -> bool:
@@ -600,7 +662,22 @@ class Scalene:
                 return False
         else:
             output = Scalene.__output
+            column_width = Scalene.__args.column_width
+            if not Scalene.__args.html:
+                # Get column width of the terminal and adjust to fit.
+                try:
+                    # If we are in a Jupyter notebook, stick with 132
+                    if "ipykernel" in sys.modules:
+                        column_width = 132
+                    else:
+                        import shutil
+
+                        column_width = shutil.get_terminal_size().columns
+                except:
+                    pass
+
             did_output: bool = output.output_profiles(
+                column_width,
                 Scalene.__stats,
                 Scalene.__pid,
                 Scalene.profile_this_code,
@@ -642,7 +719,7 @@ class Scalene:
         gpu_mem_used: float,
         prev_virtual: float,
         prev_wallclock: float,
-        prev_sys: float,
+        _prev_sys: float,
         prev_user: float,
     ) -> None:
         """Handle interrupts for CPU profiling."""
@@ -688,19 +765,25 @@ class Scalene:
         # CPU utilization is the fraction of time spent on the CPU
         # over the total time.
         elapsed_user = now_user - prev_user
-        elapsed_sys = now_sys - prev_sys
         try:
             cpu_utilization = elapsed_user / elapsed_wallclock
         except ZeroDivisionError:
             cpu_utilization = 0.0
+        # On multicore systems running multi-threaded native code, CPU
+        # utilization can exceed 1; that is, elapsed user time is
+        # longer than elapsed wallclock time. If this occurs, set
+        # wall clock time to user time and set CPU utilization to 100%.
+        if cpu_utilization > 1.0:
+            cpu_utilization = 1.0
+            elapsed_wallclock = elapsed_user
         # Deal with an odd case reported here: https://github.com/plasma-umass/scalene/issues/124
         # (Note: probably obsolete now that Scalene is using the nvidia wrappers, but just in case...)
         # We don't want to report 'nan', so turn the load into 0.
         if math.isnan(gpu_load):
             gpu_load = 0.0
-        gpu_time = gpu_load * Scalene.__last_cpu_sampling_rate
+        gpu_time = gpu_load * Scalene.__args.cpu_sampling_rate
         Scalene.__stats.total_gpu_samples += gpu_time
-        python_time = Scalene.__last_cpu_sampling_rate
+        python_time = Scalene.__args.cpu_sampling_rate
         c_time = elapsed_virtual - python_time
         if c_time < 0:
             c_time = 0
@@ -785,25 +868,6 @@ class Scalene:
 
         Scalene.__stats.total_cpu_samples += total_time
 
-        if False:
-            # Pick a new random interval, distributed around the mean.
-            next_interval = 0.0
-            while next_interval <= 0.0:
-                # Choose a normally distributed random number around the
-                # mean for the next interval. By setting the standard
-                # deviation to a fraction of the mean, we know by
-                # properties of the normal distribution that the
-                # likelihood of iterating this loop more than once is
-                # low. For a fraction 1/f, the probability is
-                # p = 1-(math.erf(f/math.sqrt(2)))/2
-                next_interval = random.normalvariate(
-                    Scalene.__args.cpu_sampling_rate,
-                    Scalene.__args.cpu_sampling_rate / 3.0,
-                )
-        else:
-            next_interval = Scalene.__args.cpu_sampling_rate
-        Scalene.__last_cpu_sampling_rate = next_interval
-
     # Returns final frame (up to a line in a file we are profiling), the thread identifier, and the original frame.
     @staticmethod
     def compute_frames_to_record(
@@ -827,7 +891,7 @@ class Scalene:
         frames.insert(
             0,
             (
-                sys._current_frames().get(tid, cast(FrameType,None)),
+                sys._current_frames().get(tid, cast(FrameType, None)),
                 tid,
             ),
         )
@@ -855,7 +919,6 @@ class Scalene:
                     frame = cast(FrameType, frame.f_back)
                     if frame:
                         fname = frame.f_code.co_filename
-                        continue
                 else:
                     break
             if frame:
@@ -870,6 +933,7 @@ class Scalene:
         """Update tracking info so we can correctly report line number info later."""
         fname = Filename(frame.f_code.co_filename)
         lineno = LineNumber(frame.f_lineno)
+
         f = frame
         try:
             while "<" in Filename(f.f_code.co_name):
@@ -912,12 +976,7 @@ class Scalene:
         stats.firstline_map[fn_name] = LineNumber(firstline)
 
     @staticmethod
-    def alloc_sigqueue_processor(
-        _signum: Union[
-            Callable[[Signals, FrameType], None], int, Handlers, None
-        ],
-        _this_frame: FrameType,
-    ) -> None:
+    def alloc_sigqueue_processor(x: Optional[List[int]]) -> None:
         """Handle interrupts for memory profiling (mallocs and frees)."""
         stats = Scalene.__stats
         curr_pid = os.getpid()
@@ -950,7 +1009,6 @@ class Scalene:
                     reported_lineno,
                     bytei_str,
                 ) = count_str.split(",")
-                # assert action in ["M", "f", "F"]
                 if int(curr_pid) == int(pid):
                     arr.append(
                         (
@@ -965,34 +1023,37 @@ class Scalene:
                         )
                     )
 
-        arr.sort()
-
-        # Iterate through the array to compute the new current footprint.
-        # and update the global __memory_footprint_samples.
-        before = stats.current_footprint
+        # Iterate through the array to compute the new current footprint
+        # and update the global __memory_footprint_samples. Since on some systems,
+        # we get free events before mallocs, force `before` to always be at least 0.
+        before = max(stats.current_footprint, 0)
         prevmax = stats.max_footprint
         freed_last_trigger = 0
-
-        for item in arr:
+        for (index, item) in enumerate(arr):
             (
                 _alloc_time,
                 action,
                 count,
-                python_fraction,
+                _python_fraction,
                 pointer,
                 fname,
                 lineno,
                 bytei,
             ) = item
-            count /= 1024 * 1024
             is_malloc = action == "M"
+            count /= 1024 * 1024
             if is_malloc:
                 stats.current_footprint += count
-                if stats.current_footprint > stats.max_footprint:
-                    stats.max_footprint = stats.current_footprint
+                stats.max_footprint = max(
+                    stats.current_footprint, stats.max_footprint
+                )
             else:
                 assert action == "f" or action == "F"
                 stats.current_footprint -= count
+                # Force current footprint to be non-negative; this
+                # code is needed because Scalene can miss some initial
+                # allocations at startup.
+                stats.current_footprint = max(0, stats.current_footprint)
                 if action == "f":
                     # Check if pointer actually matches
                     if stats.last_malloc_triggered[2] == pointer:
@@ -1007,7 +1068,7 @@ class Scalene:
                 pass
             else:
                 # We freed the last allocation trigger. Adjust scores.
-                this_fn, this_ln, this_ptr = stats.last_malloc_triggered
+                this_fn, this_ln, _this_ptr = stats.last_malloc_triggered
                 if this_ln != 0:
                     mallocs, frees = stats.leak_score[this_fn][this_ln]
                     stats.leak_score[this_fn][this_ln] = (
@@ -1038,40 +1099,57 @@ class Scalene:
                 bytei,
             ) = item
 
-            if Scalene.__last_profiled != (fname, lineno):
-                Scalene.__last_profiled_invalidated = True
+            is_malloc = action == "M"
+            if is_malloc and count == NEWLINE_TRIGGER_LENGTH + 1:
+                stats.memory_malloc_count[fname][lineno] += 1
+                stats.memory_aggregate_footprint[fname][
+                    lineno
+                ] += stats.memory_current_highwater_mark[fname][lineno]
+                stats.memory_current_footprint[fname][lineno] = 0
+                stats.memory_current_highwater_mark[fname][lineno] = 0
+                continue
 
             # Add the byte index to the set for this line (if it's not there already).
             stats.bytei_map[fname][lineno].add(bytei)
             count /= 1024 * 1024
-            is_malloc = action == "M"
             if is_malloc:
                 allocs += count
                 curr += count
                 malloc_pointer = pointer
-                stats.memory_malloc_samples[fname][lineno][bytei] += count
-                stats.memory_python_samples[fname][lineno][bytei] += (
+                stats.memory_malloc_samples[fname][lineno] += count
+                stats.memory_python_samples[fname][lineno] += (
                     python_fraction * count
                 )
                 stats.malloc_samples[fname] += 1
                 stats.total_memory_malloc_samples += count
-                # Check if we executed any other lines since the last sample.
-                if Scalene.__last_profiled_invalidated:
-                    # Yes, new line, so we bump the counter (used for later computing the average memory consumption).
-                    # print("MALLOC updating", fname, lineno, Scalene.__last_profiled, stats.memory_malloc_samples[fname][lineno][bytei])
-                    stats.memory_malloc_count[fname][lineno][bytei] += 1
-                    Scalene.__last_profiled = (fname, lineno)
-                    Scalene.__last_profiled_invalidated = False
-
-                else:
-                    # print("checked but nope: ", fname, lineno, Scalene.__last_profiled, Scalene.__last_profiled_invalidated)
-                    pass
+                # Update current and max footprints for this file & line.
+                stats.memory_current_footprint[fname][lineno] += count
+                if (
+                    stats.memory_current_footprint[fname][lineno]
+                    > stats.memory_current_highwater_mark[fname][lineno]
+                ):
+                    stats.memory_current_highwater_mark[fname][
+                        lineno
+                    ] = stats.memory_current_footprint[fname][lineno]
+                stats.memory_current_highwater_mark[fname][lineno] = max(
+                    stats.memory_current_highwater_mark[fname][lineno],
+                    stats.memory_current_footprint[fname][lineno],
+                )
+                stats.memory_max_footprint[fname][lineno] = max(
+                    stats.memory_current_footprint[fname][lineno],
+                    stats.memory_max_footprint[fname][lineno],
+                )
             else:
                 assert action == "f" or action == "F"
                 curr -= count
-                stats.memory_free_samples[fname][lineno][bytei] += count
-                stats.memory_free_count[fname][lineno][bytei] += 1
+                stats.memory_free_samples[fname][lineno] += count
+                stats.memory_free_count[fname][lineno] += 1
                 stats.total_memory_free_samples += count
+                stats.memory_current_footprint[fname][lineno] -= count
+                # Ensure that we never drop the current footprint below 0.
+                stats.memory_current_footprint[fname][lineno] = max(
+                    0, stats.memory_current_footprint[fname][lineno]
+                )
 
             stats.per_line_footprint_samples[fname][lineno].add(curr)
             # If we allocated anything, then mark this as the last triggering malloc
@@ -1081,17 +1159,17 @@ class Scalene:
                     LineNumber(lineno),
                     Address(malloc_pointer),
                 )
-        if False:
-            stats.allocation_velocity = (
-                stats.allocation_velocity[0] + (after - before),
-                stats.allocation_velocity[1] + allocs,
-            )
-            # Update leak score if we just increased the max footprint (starting at a fixed threshold, currently 100MB,
+        stats.allocation_velocity = (
+            stats.allocation_velocity[0] + (after - before),
+            stats.allocation_velocity[1] + allocs,
+        )
+        if Scalene.__args.memory_leak_detector:
+            # Update leak score if we just increased the max footprint (starting at a fixed threshold, currently 100MB
             if prevmax < stats.max_footprint and stats.max_footprint > 100:
                 stats.last_malloc_triggered = last_malloc
+                fname, lineno, _ = last_malloc
                 mallocs, frees = stats.leak_score[fname][lineno]
                 stats.leak_score[fname][lineno] = (mallocs + 1, frees)
-        del _this_frame
 
     @staticmethod
     def before_fork() -> None:
@@ -1163,23 +1241,27 @@ class Scalene:
         """Return true if the filename is one we should trace."""
         if not filename:
             return False
-        # If the @profile decorator has been used,
-        # we restrict profiling to files containing decorated functions.
-        if Scalene.__files_to_profile:
-            return filename in Scalene.__files_to_profile
-        # Generic handling follows (when no @profile decorator has been used).
-        profile_only_list = Scalene.__args.profile_only.split(",")
+        if "scalene/scalene" in filename:
+            # Don't profile the profiler.
+            return False
         if "site-packages" in filename or "/lib/python" in filename:
             # Don't profile Python internals by default.
             if not Scalene.__args.profile_all:
                 return False
+        # Generic handling follows (when no @profile decorator has been used).
+        profile_exclude_list = Scalene.__args.profile_exclude.split(",")
+        if any(
+            prof in filename for prof in profile_exclude_list if prof != ""
+        ):
+            return False
         if filename[0] == "<":
             if "<ipython" in filename:
                 # Profiling code created in a Jupyter cell:
                 # create a file to hold the contents.
-                import IPython
                 import re
 
+                import IPython
+
                 # Find the input where the function was defined;
                 # we need this to properly annotate the code.
                 result = re.match("<ipython-input-([0-9]+)-.*>", filename)
@@ -1195,13 +1277,11 @@ class Scalene:
             else:
                 # Not a real file and not a function created in Jupyter.
                 return False
-        if "scalene/scalene" in filename:
-            # Don't profile the profiler.
-            return False
         # If (a) `profile-only` was used, and (b) the file matched
         # NONE of the provided patterns, don't profile it.
-        not_found_in_profile_only = profile_only_list and not any(
-            prof in filename for prof in profile_only_list
+        profile_only_set = set(Scalene.__args.profile_only.split(","))
+        not_found_in_profile_only = profile_only_set and not any(
+            prof in filename for prof in profile_only_set
         )
         if not_found_in_profile_only:
             return False
@@ -1228,6 +1308,12 @@ class Scalene:
     def start() -> None:
         """Initiate profiling."""
         # Scalene.clear_mmap_data()
+        if not Scalene.__initialized:
+            print(
+                "ERROR: Do not try to invoke `start` when you have not called Scalene using one of the methods "
+                "in https://github.com/plasma-umass/scalene#using-scalene"
+            )
+            sys.exit(1)
         Scalene.__stats.start_clock()
         Scalene.enable_signals()
         Scalene.__done = False
@@ -1240,7 +1326,7 @@ class Scalene:
         Scalene.__stats.stop_clock()
 
     @staticmethod
-    def isDone() -> bool:
+    def is_done() -> bool:
         return Scalene.__done
 
     @staticmethod
@@ -1248,7 +1334,7 @@ class Scalene:
         _signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        _this_frame: FrameType,
+        _this_frame: Optional[FrameType],
     ) -> None:
         for pid in Scalene.__child_pids:
             os.kill(pid, Scalene.__signals.start_profiling_signal)
@@ -1259,7 +1345,7 @@ class Scalene:
         _signum: Union[
             Callable[[Signals, FrameType], None], int, Handlers, None
         ],
-        _this_frame: FrameType,
+        _this_frame: Optional[FrameType],
     ) -> None:
         for pid in Scalene.__child_pids:
             os.kill(pid, Scalene.__signals.stop_profiling_signal)
@@ -1300,7 +1386,7 @@ class Scalene:
         ],
         _this_frame: FrameType,
     ) -> None:
-        sys.exit(-1)
+        sys.exit(1)
 
     def profile_code(
         self,
@@ -1321,16 +1407,15 @@ class Scalene:
         except KeyboardInterrupt:
             # Cleanly handle keyboard interrupts (quits execution and dumps the profile).
             print("Scalene execution interrupted.")
-            pass
         except Exception as e:
             print("Error in program being profiled:\n", e)
             traceback.print_exc()
+            exit_status = 1
         finally:
             self.stop()
+            sys.settrace(None)
             # If we've collected any samples, dump them.
-            if Scalene.output_profile():
-                pass
-            else:
+            if not Scalene.output_profile():
                 print(
                     "Scalene: Program did not run for long enough to profile."
                 )
@@ -1348,17 +1433,28 @@ class Scalene:
         # the pid of the primary profiler
         Scalene.__parent_pid = args.pid if Scalene.__is_child else os.getpid()
 
+    @staticmethod
+    def set_initialized() -> None:
+        Scalene.__initialized = True
+
     @staticmethod
     def main() -> None:
         (
             args,
             left,
         ) = ScaleneParseArgs.parse_args()
+        Scalene.set_initialized()
         Scalene.run_profiler(args, left)
 
     @staticmethod
-    def run_profiler(args: argparse.Namespace, left: List[str]) -> None:
+    def run_profiler(args: argparse.Namespace, left: List[str], is_jupyter: bool = False) -> None:
         # Set up signal handlers for starting and stopping profiling.
+        if not Scalene.__initialized:
+            print(
+                "ERROR: Do not try to manually invoke `run_profiler`.\n"
+                "To invoke Scalene programmatically, see the usage noted in https://github.com/plasma-umass/scalene#using-scalene"
+            )
+            sys.exit(1)
         signal.signal(
             Scalene.__signals.start_profiling_signal,
             Scalene.start_signal_handler,
@@ -1374,8 +1470,10 @@ class Scalene:
             signal.siginterrupt(Scalene.__signals.stop_profiling_signal, False)
 
         signal.signal(signal.SIGINT, Scalene.interruption_handler)
-
-        did_preload = ScalenePreload.setup_preload(args)
+        if not is_jupyter:
+            did_preload = ScalenePreload.setup_preload(args)
+        else:
+            did_preload = False
         if not did_preload:
             with contextlib.suppress(Exception):
                 # If running in the background, print the PID.
@@ -1391,9 +1489,12 @@ class Scalene:
         Scalene.__stats.clear_all()
         sys.argv = left
         with contextlib.suppress(Exception):
-            multiprocessing.set_start_method("fork")
+            if not is_jupyter:
+                multiprocessing.set_start_method("fork")
         try:
             Scalene.process_args(args)
+            progs = None
+            exit_status = 0
             try:
                 # Look for something ending in '.py'. Treat the first one as our executable.
                 progs = [x for x in sys.argv if re.match(".*\.py$", x)]
@@ -1401,6 +1502,8 @@ class Scalene:
                 with contextlib.suppress(Exception):
                     progs.append(sys.argv[0])
                     progs.append(__file__)
+                if not progs:
+                    raise FileNotFoundError
                 with open(progs[0], "rb") as prog_being_profiled:
                     # Read in the code and compile it.
                     try:
@@ -1411,7 +1514,7 @@ class Scalene:
                         )
                     except SyntaxError:
                         traceback.print_exc()
-                        sys.exit(-1)
+                        sys.exit(1)
                     # Push the program's path.
                     program_path = os.path.dirname(os.path.abspath(progs[0]))
                     sys.path.insert(0, program_path)
@@ -1436,7 +1539,7 @@ class Scalene:
                     the_locals = __main__.__dict__
                     the_globals = __main__.__dict__
                     # Splice in the name of the file being executed instead of the profiler.
-                    the_globals["__file__"] = os.path.basename(progs[0])
+                    the_globals["__file__"] = os.path.abspath(progs[0])
                     # Some mysterious module foo to make this work the same with -m as with `scalene`.
                     the_globals["__spec__"] = None
                     # Start the profiler.
@@ -1464,21 +1567,25 @@ class Scalene:
                         print(message)
                         print(traceback.format_exc())
             except (FileNotFoundError, IOError):
-                print("Scalene: could not find input file " + progs[0])
-                sys.exit(-1)
+                if progs:
+                    print("Scalene: could not find input file " + progs[0])
+                else:
+                    print("Scalene: no input file specified.")
+                sys.exit(1)
         except SystemExit:
             pass
         except StopJupyterExecution:
             pass
         except Exception:
             print("Scalene failed to initialize.\n" + traceback.format_exc())
-            sys.exit(-1)
+            sys.exit(1)
         finally:
             with contextlib.suppress(Exception):
                 Scalene.__malloc_mapfile.close()
                 Scalene.__memcpy_mapfile.close()
                 if not Scalene.__is_child:
                     Scalene.cleanup_files()
+            sys.exit(exit_status)
 
 
 if __name__ == "__main__":
