@@ -1,17 +1,17 @@
-import cloudpickle
 import os
 import pathlib
 import pickle
 import time
-
 from collections import defaultdict
 from typing import Any, Dict, List, NewType, Set, Tuple, TypeVar
-from scalene.runningstats import RunningStats
+
+import cloudpickle
+
 from scalene.adaptive import Adaptive
+from scalene.runningstats import RunningStats
 
 Address = NewType("Address", str)
 Filename = NewType("Filename", str)
-# FunctionName = NewType("FunctionName", str)
 LineNumber = NewType("LineNumber", int)
 ByteCodeIndex = NewType("ByteCodeIndex", int)
 T = TypeVar("T")
@@ -57,13 +57,33 @@ class ScaleneStatistics:
 
         # malloc samples for each location in the program
         self.memory_malloc_samples: Dict[
-            Filename, Dict[LineNumber, Dict[ByteCodeIndex, float]]
-        ] = defaultdict(lambda: defaultdict(lambda: defaultdict(float)))
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
 
         # number of times samples were added for the above
         self.memory_malloc_count: Dict[
-            Filename, Dict[LineNumber, Dict[ByteCodeIndex, int]]
-        ] = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
+            Filename, Dict[LineNumber, int]
+        ] = defaultdict(lambda: defaultdict(int))
+
+        # the current footprint for this line
+        self.memory_current_footprint: Dict[
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
+
+        # the max footprint for this line
+        self.memory_max_footprint: Dict[
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
+
+        # the current high watermark for this line
+        self.memory_current_highwater_mark: Dict[
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
+
+        # the aggregate footprint for this line (sum of all final "current"s)
+        self.memory_aggregate_footprint: Dict[
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
 
         # the last malloc to trigger a sample (used for leak detection)
         self.last_malloc_triggered: Tuple[Filename, LineNumber, Address] = (
@@ -74,18 +94,18 @@ class ScaleneStatistics:
 
         # mallocs attributable to Python, for each location in the program
         self.memory_python_samples: Dict[
-            Filename, Dict[LineNumber, Dict[ByteCodeIndex, float]]
-        ] = defaultdict(lambda: defaultdict(lambda: defaultdict(float)))
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
 
         # free samples for each location in the program
         self.memory_free_samples: Dict[
-            Filename, Dict[LineNumber, Dict[ByteCodeIndex, float]]
-        ] = defaultdict(lambda: defaultdict(lambda: defaultdict(float)))
+            Filename, Dict[LineNumber, float]
+        ] = defaultdict(lambda: defaultdict(float))
 
         # number of times samples were added for the above
         self.memory_free_count: Dict[
-            Filename, Dict[LineNumber, Dict[ByteCodeIndex, int]]
-        ] = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
+            Filename, Dict[LineNumber, int]
+        ] = defaultdict(lambda: defaultdict(int))
 
         # memcpy samples for each location in the program
         self.memcpy_samples: Dict[
@@ -151,6 +171,10 @@ class ScaleneStatistics:
         self.malloc_samples.clear()
         self.memory_malloc_samples.clear()
         self.memory_malloc_count.clear()
+        self.memory_current_footprint.clear()
+        self.memory_max_footprint.clear()
+        self.memory_current_highwater_mark.clear()
+        self.memory_aggregate_footprint.clear()
         self.memory_python_samples.clear()
         self.memory_free_samples.clear()
         self.memory_free_count.clear()
@@ -180,11 +204,11 @@ class ScaleneStatistics:
         self.per_line_footprint_samples.clear()
 
     def start_clock(self) -> None:
-        self.start_time = time.perf_counter()
+        self.start_time = time.time()
 
     def stop_clock(self) -> None:
         if self.start_time > 0:
-            self.elapsed_time += time.perf_counter() - self.start_time
+            self.elapsed_time += time.time() - self.start_time
         self.start_time = 0
 
     def build_function_stats(self, filename: Filename):  # type: ignore
@@ -215,25 +239,25 @@ class ScaleneStatistics:
             fn_stats.per_line_footprint_samples[fn_name][
                 first_line_no
             ] += self.per_line_footprint_samples[filename][line_no]
+            fn_stats.memory_malloc_count[fn_name][
+                first_line_no
+            ] += self.memory_malloc_count[filename][line_no]
+            fn_stats.memory_free_count[fn_name][
+                first_line_no
+            ] += self.memory_free_count[filename][line_no]
+            fn_stats.memory_malloc_samples[fn_name][
+                first_line_no
+            ] += self.memory_malloc_samples[filename][line_no]
+            fn_stats.memory_python_samples[fn_name][
+                first_line_no
+            ] += self.memory_python_samples[filename][line_no]
+            fn_stats.memory_free_samples[fn_name][
+                first_line_no
+            ] += self.memory_free_samples[filename][line_no]
             for index in self.bytei_map[filename][line_no]:
                 fn_stats.bytei_map[fn_name][first_line_no].add(
                     ByteCodeIndex(0)
                 )
-                fn_stats.memory_malloc_count[fn_name][first_line_no][
-                    ByteCodeIndex(0)
-                ] += self.memory_malloc_count[filename][line_no][index]
-                fn_stats.memory_free_count[fn_name][first_line_no][
-                    ByteCodeIndex(0)
-                ] += self.memory_free_count[filename][line_no][index]
-                fn_stats.memory_malloc_samples[fn_name][first_line_no][
-                    ByteCodeIndex(0)
-                ] += self.memory_malloc_samples[filename][line_no][index]
-                fn_stats.memory_python_samples[fn_name][first_line_no][
-                    ByteCodeIndex(0)
-                ] += self.memory_python_samples[filename][line_no][index]
-                fn_stats.memory_free_samples[fn_name][first_line_no][
-                    ByteCodeIndex(0)
-                ] += self.memory_free_samples[filename][line_no][index]
             fn_stats.memcpy_samples[fn_name][
                 first_line_no
             ] += self.memcpy_samples[filename][line_no]
@@ -243,10 +267,15 @@ class ScaleneStatistics:
                 fn_stats.leak_score[fn_name][first_line_no][1]
                 + self.leak_score[filename][line_no][1],
             )
+            fn_stats.memory_max_footprint[fn_name][first_line_no] = max(
+                fn_stats.memory_max_footprint[fn_name][first_line_no],
+                self.memory_max_footprint[filename][line_no],
+            )
         return fn_stats
 
     payload_contents = [
         "max_footprint",
+        "current_footprint",
         "elapsed_time",
         "total_cpu_samples",
         "cpu_samples_c",
@@ -303,18 +332,6 @@ class ScaleneStatistics:
             for lineno in src[filename]:
                 dest[filename][lineno] += src[filename][lineno]
 
-    @staticmethod
-    def increment_per_bytecode_samples(
-        dest: Dict[Filename, Dict[LineNumber, Dict[ByteCodeIndex, T]]],
-        src: Dict[Filename, Dict[LineNumber, Dict[ByteCodeIndex, T]]],
-    ) -> None:
-        for filename in src:
-            for lineno in src[filename]:
-                for ind in src[filename][lineno]:
-                    dest[filename][lineno][ind] += src[  # type: ignore
-                        filename
-                    ][lineno][ind]
-
     def merge_stats(self, the_dir_name: pathlib.Path) -> None:
         the_dir = pathlib.Path(the_dir_name)
         for f in list(the_dir.glob("**/scalene*")):
@@ -328,6 +345,9 @@ class ScaleneStatistics:
                 for i, n in enumerate(ScaleneStatistics.payload_contents):
                     setattr(x, n, value[i])
                 self.max_footprint = max(self.max_footprint, x.max_footprint)
+                self.current_footprint = max(
+                    self.current_footprint, x.current_footprint
+                )
                 self.increment_cpu_utilization(
                     self.cpu_utilization, x.cpu_utilization
                 )
@@ -350,25 +370,29 @@ class ScaleneStatistics:
                     self.per_line_footprint_samples,
                     x.per_line_footprint_samples,
                 )
-                self.increment_per_bytecode_samples(
-                    self.memory_malloc_samples, x.memory_malloc_samples
-                )
-                self.increment_per_bytecode_samples(
-                    self.memory_python_samples, x.memory_python_samples
-                )
-                self.increment_per_bytecode_samples(
-                    self.memory_free_samples, x.memory_free_samples
-                )
-                self.increment_per_bytecode_samples(
+                self.increment_per_line_samples(
                     self.memory_malloc_count, x.memory_malloc_count
                 )
-                self.increment_per_bytecode_samples(
+                self.increment_per_line_samples(
+                    self.memory_malloc_samples, x.memory_malloc_samples
+                )
+                self.increment_per_line_samples(
+                    self.memory_python_samples, x.memory_python_samples
+                )
+                self.increment_per_line_samples(
+                    self.memory_free_samples, x.memory_free_samples
+                )
+                self.increment_per_line_samples(
                     self.memory_free_count, x.memory_free_count
                 )
                 for filename in x.bytei_map:
                     for lineno in x.bytei_map[filename]:
                         v = x.bytei_map[filename][lineno]
                         self.bytei_map[filename][lineno] |= v
+                        self.memory_max_footprint[filename][lineno] = max(
+                            self.memory_max_footprint[filename][lineno],
+                            x.memory_max_footprint[filename][lineno],
+                        )
                 for filename in x.cpu_samples:
                     self.cpu_samples[filename] += x.cpu_samples[filename]
                 self.total_memory_free_samples += x.total_memory_free_samples
